@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2007 West Consulting
-   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Arthur de Jong
+   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -85,6 +85,8 @@ struct ldap_config
 {
   /* the number of threads to start */
   int ldc_threads;
+  /* the user name specified in the uid option */
+  char *ldc_uidname;
   /* the user id nslcd should be run as */
   uid_t ldc_uid;
   /* the group id nslcd should be run as */
@@ -113,6 +115,10 @@ struct ldap_config
   char *ldc_sasl_authzid;
   /* sasl security */
   char *ldc_sasl_secprops;
+#ifdef LDAP_OPT_X_SASL_NOCANON
+  /* whether host name should be canonicalised */
+  int ldc_sasl_canonicalize;
+#endif /* LDAP_OPT_X_SASL_NOCANON */
   /* base DN, eg. dc=gnu,dc=org */
   const char *ldc_bases[NSS_LDAP_CONFIG_MAX_BASES];
   /* scope for searches */
@@ -143,10 +149,13 @@ struct ldap_config
   SET *ldc_nss_initgroups_ignoreusers;
   /* the searches that should be performed to do autorisation checks */
   char *ldc_pam_authz_search[NSS_LDAP_CONFIG_MAX_AUTHZ_SEARCHES];
-  /* minimum uid for users retreived from LDAP */
+  /* minimum uid for users retrieved from LDAP */
   uid_t ldc_nss_min_uid;
   /* the regular expression to determine valid names */
   regex_t validnames;
+  /* whether password changing should be denied and user prompted with
+     this message */
+  char *pam_password_prohibit_message;
 };
 
 /* this is a pointer to the global configuration, it should be available
