@@ -3,7 +3,7 @@
    This file is part of the nss-pam-ldapd library.
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,10 @@
 
 #include <errno.h>
 #include <limits.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif /* HAVE_STDINT_H */
+#include <sys/types.h>
 
 #include "nslcd.h"
 #include "common/nslcd-prot.h"
@@ -117,7 +121,7 @@ int update_lastchange(MYLDAP_SESSION *session,const char *userdn);
    requested*/
 MYLDAP_ENTRY *shadow_uid2entry(MYLDAP_SESSION *session,const char *username,int *rcp);
 
-/* return shadown information */
+/* return shadow information */
 void get_shadow_properties(MYLDAP_ENTRY *entry,long *lastchangedate,
                            long *mindays,long *maxdays,long *warndays,
                            long *inactdays,long *expiredate,unsigned long *flag);
@@ -182,6 +186,7 @@ void shadow_init(void);
 
 /* these are the different functions that handle the database
    specific actions, see nslcd.h for the action descriptions */
+int nslcd_config_get(TFILE *fp,MYLDAP_SESSION *session);
 int nslcd_alias_byname(TFILE *fp,MYLDAP_SESSION *session);
 int nslcd_alias_all(TFILE *fp,MYLDAP_SESSION *session);
 int nslcd_ether_byname(TFILE *fp,MYLDAP_SESSION *session);
