@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2006 Luke Howard
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -170,6 +170,11 @@ void group_init(void)
   attmap_add_attributes(set,attmap_group_gidNumber);
   attmap_add_attributes(set,attmap_group_member);
   group_attrs=set_tolist(set);
+  if (group_attrs==NULL)
+  {
+    log_log(LOG_CRIT,"malloc() failed to allocate memory");
+    exit(EXIT_FAILURE);
+  }
   set_free(set);
 }
 
@@ -318,7 +323,7 @@ static int write_group(TFILE *fp,MYLDAP_ENTRY *entry,const char *reqname,
 NSLCD_HANDLE(
   group,byname,
   char name[256];
-  char filter[1024];
+  char filter[4096];
   READ_STRING(fp,name);
   log_setrequest("group=\"%s\"",name);
   if (!isvalidname(name)) {
@@ -333,7 +338,7 @@ NSLCD_HANDLE(
 NSLCD_HANDLE(
   group,bygid,
   gid_t gid;
-  char filter[1024];
+  char filter[4096];
   READ_TYPE(fp,gid,gid_t);
   log_setrequest("group=%d",(int)gid);,
   NSLCD_ACTION_GROUP_BYGID,
@@ -344,7 +349,7 @@ NSLCD_HANDLE(
 NSLCD_HANDLE(
   group,bymember,
   char name[256];
-  char filter[1024];
+  char filter[4096];
   READ_STRING(fp,name);
   log_setrequest("group/member=\"%s\"",name);
   if (!isvalidname(name))
