@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2006 Luke Howard
    Copyright (C) 2006, 2007 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -413,7 +413,6 @@ static int do_sasl_interact(LDAP UNUSED(*ld),unsigned UNUSED(flags),void *defaul
 static int do_bind(LDAP *ld,const char *binddn,const char *bindpw,const char *uri)
 {
   int rc;
-  char *msg=NULL;
 #ifdef HAVE_LDAP_SASL_INTERACTIVE_BIND_S
 #ifndef HAVE_SASL_INTERACT_T
   struct berval cred;
@@ -428,18 +427,13 @@ static int do_bind(LDAP *ld,const char *binddn,const char *bindpw,const char *ur
     rc=ldap_start_tls_s(ld,NULL,NULL);
     if (rc!=LDAP_SUCCESS)
     {
-#ifdef LDAP_OPT_DIAGNOSTIC_MESSAGE
-      ldap_get_option(ld,LDAP_OPT_DIAGNOSTIC_MESSAGE,&msg);
-#endif /* LDAP_OPT_DIAGNOSTIC_MESSAGE */
       myldap_err(LOG_WARNING,ld,rc,"ldap_start_tls_s() failed (uri=%s)",uri);
-      if (msg)
-        ldap_memfree(msg);
       return rc;
     }
   }
 #endif /* LDAP_OPT_X_TLS */
   /* check if the binddn and bindpw are overwritten in the session */
-  if ((binddn!=NULL)&(binddn[0]!='\0'))
+  if ((binddn!=NULL)&&(binddn[0]!='\0'))
   {
     /* do a simple bind */
     log_log(LOG_DEBUG,"ldap_simple_bind_s(\"%s\",%s) (uri=\"%s\")",binddn,
@@ -503,6 +497,7 @@ static int do_bind(LDAP *ld,const char *binddn,const char *bindpw,const char *ur
 #ifdef HAVE_LDAP_SET_REBIND_PROC
 /* This function is called by the LDAP library when chasing referrals.
    It is configured with the ldap_set_rebind_proc() below. */
+#if defined(LDAP_API_FEATURE_X_OPENLDAP) && (LDAP_API_VERSION > 2000)
 static int do_rebind(LDAP *ld,LDAP_CONST char *url,
                      ber_tag_t UNUSED(request),
                      ber_int_t UNUSED(msgid),void *arg)
@@ -511,6 +506,34 @@ static int do_rebind(LDAP *ld,LDAP_CONST char *url,
   log_log(LOG_DEBUG,"rebinding to %s",url);
   return do_bind(ld,session->binddn,session->bindpw,url);
 }
+#else /* not recent OpenLDAP */
+static int do_rebind(LDAP *ld,char **dnp,char **passwdp,int *authmethodp,
+                     int freeit,void *arg)
+{
+  MYLDAP_SESSION *session=(MYLDAP_SESSION *)arg;
+  if (freeit)
+  {
+    free(*dnp);
+    memset(*passwdp,0,strlen(*passwdp));
+    free(*passwdp);
+  }
+  else
+  {
+    log_log(LOG_DEBUG,"rebinding");
+    *dnp=strdup(session->binddn);
+    *passwdp=strdup(session->bindpw);
+    *authmethodp=LDAP_AUTH_SIMPLE;
+    if ((*dnp==NULL)||(*passwdp==NULL))
+    {
+      if (*dnp!=NULL)
+        free(*dnp);
+      log_log(LOG_CRIT,"do_rebind(): strdup() failed to allocate memory");
+      return LDAP_NO_MEMORY;
+    }
+  }
+  return LDAP_SUCCESS;
+}
+#endif /* not recent OpenLDAP */
 #endif /* HAVE_LDAP_SET_REBIND_PROC */
 
 /* set a recieve and send timeout on a socket */
@@ -548,7 +571,7 @@ static int set_socket_timeout(LDAP *ld,time_t sec,suseconds_t usec)
 
 #ifdef LDAP_OPT_CONNECT_CB
 /* This function is called by the LDAP library once a connection was made to the server. We
-   set a timeout on the socket here, to catch netzwork timeouts during the ssl
+   set a timeout on the socket here, to catch network timeouts during the ssl
    handshake phase. It is configured with LDAP_OPT_CONNECT_CB. */
 static int connect_cb(LDAP *ld,Sockbuf UNUSED(*sb),LDAPURLDesc UNUSED(*srv),
         struct sockaddr UNUSED(*addr),struct ldap_conncb UNUSED(*ctx))
@@ -644,6 +667,13 @@ static int do_set_options(MYLDAP_SESSION *session)
     LDAP_SET_OPTION(session->ld,LDAP_OPT_X_TLS,&i);
   }
 #endif /* LDAP_OPT_X_TLS */
+#ifdef LDAP_OPT_X_SASL_NOCANON
+  if (nslcd_cfg->ldc_sasl_canonicalize>=0)
+  {
+    log_log(LOG_DEBUG,"ldap_set_option(LDAP_OPT_X_SASL_NOCANON,%s)",nslcd_cfg->ldc_sasl_canonicalize?"LDAP_OPT_OFF":"LDAP_OPT_ON");
+    LDAP_SET_OPTION(session->ld,LDAP_OPT_X_SASL_NOCANON,nslcd_cfg->ldc_sasl_canonicalize?LDAP_OPT_OFF:LDAP_OPT_ON);
+  }
+#endif /* LDAP_OPT_X_SASL_NOCANON */
   /* if nothing above failed, everything should be fine */
   return LDAP_SUCCESS;
 }
@@ -939,13 +969,13 @@ static int do_retry_search(MYLDAP_SEARCH *search)
         rc=do_try_search(search);
         if (rc==LDAP_SUCCESS)
         {
-          /* update ok time */
           pthread_mutex_lock(&uris_mutex);
-          current_uri->firstfail=0;
-          current_uri->lastfail=0;
           /* check if we are coming back from an error */
           if ((current_uri->lastfail>0)||(search->session->current_uri!=start_uri))
             log_log(LOG_INFO,"connected to LDAP server %s",current_uri->uri);
+          /* update ok time */
+          current_uri->firstfail=0;
+          current_uri->lastfail=0;
           pthread_mutex_unlock(&uris_mutex);
           /* flag the search as valid */
           search->valid=1;
@@ -1318,7 +1348,7 @@ MYLDAP_ENTRY *myldap_get_entry(MYLDAP_SEARCH *search,int *rcp)
 
 /* Get the DN from the entry. This function only returns NULL (and sets
    errno) if an incorrect entry is passed. If the DN value cannot be
-   retreived "unknown" is returned instead. */
+   retrieved "unknown" is returned instead. */
 const char *myldap_get_dn(MYLDAP_ENTRY *entry)
 {
   int rc;
@@ -1329,7 +1359,7 @@ const char *myldap_get_dn(MYLDAP_ENTRY *entry)
     errno=EINVAL;
     return "unknown";
   }
-  /* if we don't have it yet, retreive it */
+  /* if we don't have it yet, retrieve it */
   if ((entry->dn==NULL)&&(entry->search->valid))
   {
     entry->dn=ldap_get_dn(entry->search->session->ld,entry->search->msg);
@@ -1539,14 +1569,13 @@ static const char **bervalues_to_values(struct berval **bvalues)
   for (i=0;i<num_values;i++)
     sz+=bvalues[i]->bv_len+1;
   /* allocate the needed memory */
-  buf=(char *)malloc(sz);
-  if (buf==NULL)
+  values=(char **)malloc(sz);
+  if (values==NULL)
   {
-    log_log(LOG_CRIT,"myldap_get_values_len(): malloc() failed to allocate memory");
-    ldap_value_free_len(bvalues);
+    log_log(LOG_CRIT,"bervalues_to_values(): malloc() failed to allocate memory");
     return NULL;
   }
-  values=(char **)buf;
+  buf=(char *)values;
   buf+=(num_values+1)*sizeof(char *);
   /* copy from bvalues */
   for (i=0;i<num_values;i++)
@@ -1595,6 +1624,7 @@ const char **myldap_get_values_len(MYLDAP_ENTRY *entry,const char *attr)
       rc=LDAP_SUCCESS;
       if (ldap_set_option(entry->search->session->ld,LDAP_OPT_ERROR_NUMBER,&rc)!=LDAP_SUCCESS)
         log_log(LOG_WARNING,"failed to clear the error flag");
+      return NULL;
     }
     else if (rc==LDAP_SUCCESS)
     {
@@ -1604,18 +1634,29 @@ const char **myldap_get_values_len(MYLDAP_ENTRY *entry,const char *attr)
       if (set==NULL)
         return NULL;
       values=set_tolist(set);
+      set_free(set);
+      if (values==NULL)
+      {
+        log_log(LOG_CRIT,"myldap_get_values_len(): malloc() failed to allocate memory");
+        return NULL;
+      }
     }
     else
+    {
       myldap_err(LOG_WARNING,entry->search->session->ld,rc,
                  "myldap_get_values_len() of attribute \"%s\" on entry \"%s\" returned NULL",
                  attr,myldap_get_dn(entry));
-    return NULL;
+      return NULL;
+    }
   }
   else
   {
     values=bervalues_to_values(bvalues);
     ldap_value_free_len(bvalues);
   }
+  /* check if we got allocated memory */
+  if (values==NULL)
+    return NULL;
   /* store values entry so we can free it later on */
   for (i=0;i<MAX_RANGED_ATTRIBUTES_PER_ENTRY;i++)
     if (entry->rangedattributevalues[i]==NULL)
