@@ -2,7 +2,7 @@
    pam.c - pam processing routines
 
    Copyright (C) 2009 Howard Chu
-   Copyright (C) 2009, 2010, 2011, 2012 Arthur de Jong
+   Copyright (C) 2009, 2010, 2011, 2012, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -154,7 +154,7 @@ static int check_shadow(MYLDAP_SESSION *session,const char *username,
     return NSLCD_PAM_SUCCESS; /* no shadow entry found, nothing to check */
   /* get today's date */
   today=(long)(time(NULL)/(60*60*24));
-  /* get shadown information */
+  /* get shadow information */
   get_shadow_properties(entry,&lastchangedate,&mindays,&maxdays,&warndays,
                         &inactdays,&expiredate,&flag);
   /* check account expiry date */
@@ -379,7 +379,7 @@ static int try_autzsearch(MYLDAP_SESSION *session,const char *dn,
   char hostname[HOST_NAME_MAX+1];
   const char *fqdn;
   DICT *dict=NULL;
-  char filter_buffer[4096];
+  char filter[4096];
   MYLDAP_SEARCH *search;
   MYLDAP_ENTRY *entry;
   static const char *attrs[2];
@@ -409,7 +409,7 @@ static int try_autzsearch(MYLDAP_SESSION *session,const char *dn,
     }
     /* build the search filter */
     res=expr_parse(nslcd_cfg->ldc_pam_authz_search[i],
-                   filter_buffer,sizeof(filter_buffer),
+                   filter,sizeof(filter),
                    autzsearch_var_get,(void *)dict);
     if (res==NULL)
     {
@@ -418,19 +418,19 @@ static int try_autzsearch(MYLDAP_SESSION *session,const char *dn,
       log_log(LOG_ERR,"invalid pam_authz_search \"%s\"",nslcd_cfg->ldc_pam_authz_search[i]);
       return LDAP_LOCAL_ERROR;
     }
-    log_log(LOG_DEBUG,"trying pam_authz_search \"%s\"",filter_buffer);
+    log_log(LOG_DEBUG,"trying pam_authz_search \"%s\"",filter);
     /* perform the search */
     attrs[0]="dn";
     attrs[1]=NULL;
     /* FIXME: this only searches the first base */
     search=myldap_search(session,nslcd_cfg->ldc_bases[0],LDAP_SCOPE_SUBTREE,
-                         filter_buffer,attrs,&rc);
+                         filter,attrs,&rc);
     if (search==NULL)
     {
       autzsearch_vars_free(dict);
       dict_free(dict);
       log_log(LOG_ERR,"pam_authz_search \"%s\" failed: %s",
-              filter_buffer,ldap_err2string(rc));
+              filter,ldap_err2string(rc));
       return rc;
     }
     /* try to get an entry */
@@ -439,7 +439,7 @@ static int try_autzsearch(MYLDAP_SESSION *session,const char *dn,
     {
       autzsearch_vars_free(dict);
       dict_free(dict);
-      log_log(LOG_ERR,"pam_authz_search \"%s\" found no matches",filter_buffer);
+      log_log(LOG_ERR,"pam_authz_search \"%s\" found no matches",filter);
       if (rc==LDAP_SUCCESS)
         rc=LDAP_NO_SUCH_OBJECT;
       return rc;
@@ -574,7 +574,8 @@ int nslcd_pam_sess_c(TFILE *fp,MYLDAP_SESSION *session)
 }
 
 /* perform an LDAP password modification, returns an LDAP status code */
-static int try_pwmod(const char *binddn,const char *userdn,
+static int try_pwmod(MYLDAP_SESSION *oldsession,
+                     const char *binddn,const char *userdn,
                      const char *oldpassword,const char *newpassword)
 {
   MYLDAP_SESSION *session;
@@ -597,7 +598,9 @@ static int try_pwmod(const char *binddn,const char *userdn,
     if (rc==LDAP_SUCCESS)
     {
       /* try to update the shadowLastChange attribute */
-      (void)update_lastchange(session,userdn);
+      if (update_lastchange(session,userdn)!=LDAP_SUCCESS)
+        /* retry with the normal session */
+        (void)update_lastchange(oldsession,userdn);
     }
   }
   /* close the session */
@@ -647,6 +650,18 @@ int nslcd_pam_pwmod(TFILE *fp,MYLDAP_SESSION *session,uid_t calleruid)
     }
     return -1;
   }
+  /* check if pam_password_prohibit_message is set */
+  if (nslcd_cfg->pam_password_prohibit_message!=NULL)
+  {
+    log_log(LOG_NOTICE,"password change prohibited");
+    WRITE_INT32(fp,NSLCD_RESULT_BEGIN);
+    WRITE_STRING(fp,username);
+    WRITE_STRING(fp,"");
+    WRITE_INT32(fp,NSLCD_PAM_PERM_DENIED);
+    WRITE_STRING(fp,nslcd_cfg->pam_password_prohibit_message);
+    WRITE_INT32(fp,NSLCD_RESULT_END);
+    return 0;
+  }
   /* check if the the user passed the rootpwmoddn */
   if (asroot)
   {
@@ -679,7 +694,7 @@ int nslcd_pam_pwmod(TFILE *fp,MYLDAP_SESSION *session,uid_t calleruid)
     }
   }
   /* perform password modification */
-  rc=try_pwmod(binddn,myldap_get_dn(entry),oldpassword,newpassword);
+  rc=try_pwmod(session,binddn,myldap_get_dn(entry),oldpassword,newpassword);
   if (rc!=LDAP_SUCCESS)
   {
     mysnprintf(authzmsg,sizeof(authzmsg)-1,"password change failed: %s",ldap_err2string(rc));
