@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -155,6 +155,11 @@ void passwd_init(void)
   attmap_add_attributes(set,attmap_passwd_homeDirectory);
   attmap_add_attributes(set,attmap_passwd_loginShell);
   passwd_attrs=set_tolist(set);
+  if (passwd_attrs==NULL)
+  {
+    log_log(LOG_CRIT,"malloc() failed to allocate memory");
+    exit(EXIT_FAILURE);
+  }
   set_free(set);
 }
 
@@ -324,7 +329,7 @@ char *dn2uid(MYLDAP_SESSION *session,const char *dn,char *buf,size_t buflen)
     /* copy the uid if needed */
     if (cacheentry->uid==NULL)
       cacheentry->uid=uid!=NULL?strdup(uid):NULL;
-    else if (strcmp(cacheentry->uid,uid)!=0)
+    else if ((uid==NULL)||(strcmp(cacheentry->uid,uid)!=0))
     {
       free(cacheentry->uid);
       cacheentry->uid=uid!=NULL?strdup(uid):NULL;
@@ -342,7 +347,7 @@ MYLDAP_ENTRY *uid2entry(MYLDAP_SESSION *session,const char *uid,int *rcp)
   const char *base;
   int i;
   static const char *attrs[3];
-  char filter[1024];
+  char filter[4096];
   /* if it isn't a valid username, just bail out now */
   if (!isvalidname(uid))
   {
@@ -385,6 +390,14 @@ char *uid2dn(MYLDAP_SESSION *session,const char *uid,char *buf,size_t buflen)
   return myldap_cpy_dn(entry,buf,buflen);
 }
 
+#ifndef NSS_FLAVOUR_GLIBC
+
+/* only check nsswitch.conf for glibc */
+#define check_nsswitch_reload()
+#define shadow_uses_ldap() (1)
+
+#else /* NSS_FLAVOUR_GLIBC */
+
 /* the cached value of whether shadow lookups use LDAP in nsswitch.conf */
 #define NSSWITCH_FILE "/etc/nsswitch.conf"
 #define CACHED_UNKNOWN 22
@@ -431,6 +444,8 @@ static inline int shadow_uses_ldap(void)
   return cached_shadow_uses_ldap;
 }
 
+#endif /* NSS_FLAVOUR_GLIBC */
+
 /* the maximum number of uidNumber attributes per entry */
 #define MAXUIDS_PER_ENTRY 5
 
@@ -587,7 +602,7 @@ static int write_passwd(TFILE *fp,MYLDAP_ENTRY *entry,const char *requser,
 NSLCD_HANDLE_UID(
   passwd,byname,
   char name[256];
-  char filter[1024];
+  char filter[4096];
   READ_STRING(fp,name);
   log_setrequest("passwd=\"%s\"",name);
   if (!isvalidname(name)) {
@@ -603,7 +618,7 @@ NSLCD_HANDLE_UID(
 NSLCD_HANDLE_UID(
   passwd,byuid,
   uid_t uid;
-  char filter[1024];
+  char filter[4096];
   READ_TYPE(fp,uid,uid_t);
   log_setrequest("passwd=%d",(int)uid);
   if (uid<nslcd_cfg->ldc_nss_min_uid)
@@ -612,6 +627,7 @@ NSLCD_HANDLE_UID(
     WRITE_INT32(fp,NSLCD_VERSION);
     WRITE_INT32(fp,NSLCD_ACTION_PASSWD_BYUID);
     WRITE_INT32(fp,NSLCD_RESULT_END);
+    return 0;
   }
   check_nsswitch_reload();,
   NSLCD_ACTION_PASSWD_BYUID,
