@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2009, 2010, 2011, 2012 Arthur de Jong
+   Copyright (C) 2006, 2007, 2009, 2010, 2011, 2012, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -28,7 +28,9 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif /* HAVE_STDINT_H */
 
 #include "common.h"
 #include "log.h"
@@ -152,7 +154,7 @@ static int write_service(TFILE *fp,MYLDAP_ENTRY *entry,
   /* if the service name is not yet found, get the first entry */
   if (name==NULL)
     name=aliases[0];
-  /* check case of returned servies entry */
+  /* check case of returned services entry */
   if ((reqname!=NULL)&&(STR_CMP(reqname,name)!=0))
   {
     for (i=0;(aliases[i]!=NULL)&&(STR_CMP(reqname,aliases[i])!=0);i++)
@@ -213,10 +215,10 @@ NSLCD_HANDLE(
   service,byname,
   char name[256];
   char protocol[256];
-  char filter[1024];
+  char filter[4096];
   READ_STRING(fp,name);
   READ_STRING(fp,protocol);
-  log_setrequest("service=\"%s\"/%s",name,protocol);,
+  log_setrequest("service=\"%s\"%s%s",name,*protocol!='\0'?"/":"",protocol);,
   NSLCD_ACTION_SERVICE_BYNAME,
   mkfilter_service_byname(name,protocol,filter,sizeof(filter)),
   write_service(fp,entry,name,protocol)
@@ -226,10 +228,10 @@ NSLCD_HANDLE(
   service,bynumber,
   int number;
   char protocol[256];
-  char filter[1024];
+  char filter[4096];
   READ_INT32(fp,number);
   READ_STRING(fp,protocol);
-  log_setrequest("service=%d/%s",number,protocol);,
+  log_setrequest("service=%lu%s%s",(unsigned long int)number,*protocol!='\0'?"/":"",protocol);,
   NSLCD_ACTION_SERVICE_BYNUMBER,
   mkfilter_service_bynumber(number,protocol,filter,sizeof(filter)),
   write_service(fp,entry,NULL,protocol)
