@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -107,6 +107,11 @@ void shadow_init(void)
   attmap_add_attributes(set,attmap_shadow_shadowExpire);
   attmap_add_attributes(set,attmap_shadow_shadowFlag);
   shadow_attrs=set_tolist(set);
+  if (shadow_attrs==NULL)
+  {
+    log_log(LOG_CRIT,"malloc() failed to allocate memory");
+    exit(EXIT_FAILURE);
+  }
   set_free(set);
 }
 
@@ -328,7 +333,7 @@ MYLDAP_ENTRY *shadow_uid2entry(MYLDAP_SESSION *session,const char *username,int
   MYLDAP_SEARCH *search=NULL;
   MYLDAP_ENTRY *entry=NULL;
   const char *base;
-  char filter[1024];
+  char filter[4096];
   int i;
   /* if it isn't a valid username, just bail out now */
   if (!isvalidname(username))
@@ -360,7 +365,7 @@ MYLDAP_ENTRY *shadow_uid2entry(MYLDAP_SESSION *session,const char *username,int
 NSLCD_HANDLE(
   shadow,byname,
   char name[256];
-  char filter[1024];
+  char filter[4096];
   READ_STRING(fp,name);
   log_setrequest("shadow=\"%s\"",name);,
   NSLCD_ACTION_SHADOW_BYNAME,
