@@ -1,7 +1,7 @@
 /*
    common.c - common definitions
 
-   Copyright (C) 2010 Arthur de Jong
+   Copyright (C) 2010, 2013 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -19,4 +19,11 @@
    02110-1301 USA
 */
 
+#include "config.h"
+
+#include <stddef.h>
+
 int _nss_ldap_enablelookups=1;
+
+/* version information about the NSS module */
+char *_nss_ldap_version[3]={PACKAGE, VERSION, NULL};
