@@ -2,7 +2,7 @@
    ethers.c - NSS lookup functions for ethers database
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2010 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2010, 2012 Arthur de Jong
    Copyright (C) 2010 Symas Corporation
 
    This library is free software; you can redistribute it and/or
@@ -37,6 +37,7 @@ static nss_status_t read_etherent(
 {
   int32_t tmpint32;
   size_t bufptr=0;
+  memset(result,0,sizeof(struct etherent));
   READ_BUF_STRING(fp,result->e_name);
   READ_TYPE(fp,result->e_addr,uint8_t[6]);
   return NSS_STATUS_SUCCESS;
@@ -92,61 +93,70 @@ nss_status_t _nss_ldap_endetherent(void)
 
 #ifdef NSS_FLAVOUR_SOLARIS
 
+/* we disable NSS_BUFCHECK because these functions do not use the buffer */
+#undef NSS_BUFCHECK
+#define NSS_BUFCHECK ;
+
+/* provide a fallback definition */
 #ifndef NSS_BUFLEN_ETHERS
-#define NSS_BUFLEN_ETHERS 1024
+#define NSS_BUFLEN_ETHERS HOST_NAME_MAX
 #endif /* NSS_BUFLEN_ETHERS */
 
 #ifdef HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN
+static char *etherent2str(struct etherent *result,char *buffer,size_t buflen)
+{
+  int res;
+  res=snprintf(buffer,buflen,"%s %s",ether_ntoa(&result->e_addr),result->e_name);
+  if ((res<0)||(res>=buflen))
+    return NULL;
+  return buffer;
+}
+#endif /* HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
 
-static nss_status_t read_etherstring(TFILE *fp,nss_XbyY_args_t *args)
+static nss_status_t read_result(TFILE *fp,nss_XbyY_args_t *args,int wantname)
 {
   struct etherent result;
+  char buffer[NSS_BUFLEN_ETHERS];
   nss_status_t retv;
-  char *buffer;
-  int res;
-  /* read the etherent into a temporary buffer */
-  buffer=(char *)malloc(args->buf.buflen);
-  if (buffer==NULL)
-    return NSS_STATUS_UNAVAIL;
-  retv=read_etherent(fp,&result,buffer,args->buf.buflen,&errno);
+  /* read the result entry from the stream */
+  retv=read_etherent(fp,&result,buffer,sizeof(buffer),&args->erange);
   if (retv!=NSS_STATUS_SUCCESS)
-  {
-    free(buffer);
     return retv;
+#ifdef HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN
+  /* try to return in string format if requested */
+  if ((args->buf.buffer!=NULL)&&(args->buf.buflen>0))
+  {
+    if (etherent2str(&result,args->buf.buffer,args->buf.buflen)==NULL)
+    {
+      args->erange=1;
+      return NSS_NOTFOUND;
+    }
+    args->returnval=args->buf.buffer;
+    args->returnlen=strlen(args->returnval);
+    return NSS_SUCCESS;
   }
-  /* make a string representation */
-  res=snprintf(args->buf.buffer,args->buf.buflen,
-               "%s %s",ether_ntoa(&result.e_addr),result.e_name);
-  free(buffer);
-  if ((res<0)||(res>=args->buf.buflen))
-    return NSS_STATUS_TRYAGAIN;
-  NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.buffer;
-  NSS_ARGS(args)->returnlen=strlen(NSS_ARGS(args)->buf.buffer);
-  return NSS_STATUS_SUCCESS;
+#endif /* HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
+  /* return the result entry */
+  if (wantname)
+  {
+    /* we expect the buffer to have enough room for the name (buflen==0) */
+    strcpy(args->buf.buffer,result.e_name);
+    args->returnval=args->buf.buffer;
+  }
+  else /* address */
+  {
+    memcpy(args->buf.result,&result.e_addr,sizeof(result.e_addr));
+    args->returnval=args->buf.result;
+  }
+  return NSS_SUCCESS;
 }
 
-#define READ_RESULT(fp) \
-  NSS_ARGS(args)->buf.result? \
-    read_etherent(fp,(struct etherent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno): \
-    read_etherstring(fp,args); \
-  if ((NSS_ARGS(args)->buf.result)&&(retv==NSS_STATUS_SUCCESS)) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#else /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
-
-#define READ_RESULT(fp) \
-  read_etherent(fp,(struct etherent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno); \
-  if (retv==NSS_STATUS_SUCCESS) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#endif /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
-
 /* map a hostname to the corresponding ethernet address */
 static nss_status_t ethers_gethostton(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYNAME(NSLCD_ACTION_ETHER_BYNAME,
              NSS_ARGS(args)->key.name,
-             READ_RESULT(fp));
+             read_result(fp,args,0));
 }
 
 /* map an ethernet address to the corresponding hostname */
@@ -155,17 +165,11 @@ static nss_status_t ethers_getntohost(nss_backend_t UNUSED(*be),void *args)
   struct ether_addr *addr=(struct ether_addr *)(NSS_ARGS(args)->key.ether);
   NSS_BYTYPE(NSLCD_ACTION_ETHER_BYETHER,
              *addr,uint8_t[6],
-             READ_RESULT(fp));
-}
-
-static nss_status_t ethers_destructor(nss_backend_t *be,void UNUSED(*args))
-{
-  free(be);
-  return NSS_STATUS_SUCCESS;
+             read_result(fp,args,1));
 }
 
 static nss_backend_op_t ethers_ops[]={
-  ethers_destructor,
+  nss_ldap_destructor,
   ethers_gethostton,
   ethers_getntohost
 };
@@ -173,12 +177,7 @@ static nss_backend_op_t ethers_ops[]={
 nss_backend_t *_nss_ldap_ethers_constr(const char UNUSED(*db_name),
                   const char UNUSED(*src_name),const char UNUSED(*cfg_args))
 {
-  nss_backend_t *be;
-  if (!(be=(nss_backend_t *)malloc(sizeof(*be))))
-    return NULL;
-  be->ops=ethers_ops;
-  be->n_ops=sizeof(ethers_ops)/sizeof(nss_backend_op_t);
-  return be;
+  return nss_ldap_constructor(ethers_ops,sizeof(ethers_ops));
 }
 
 #endif /* NSS_FLAVOUR_SOLARIS */
