@@ -2,7 +2,7 @@
    group.c - NSS lookup functions for group database
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2009, 2010 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2009, 2010, 2012 Arthur de Jong
    Copyright (C) 2010 Symas Corporation
 
    This library is free software; you can redistribute it and/or
@@ -38,6 +38,7 @@ static nss_status_t read_group(
 {
   int32_t tmpint32,tmp2int32,tmp3int32;
   size_t bufptr=0;
+  memset(result,0,sizeof(struct group));
   READ_BUF_STRING(fp,result->gr_name);
   READ_BUF_STRING(fp,result->gr_passwd);
   READ_TYPE(fp,result->gr_gid,gid_t);
@@ -72,11 +73,11 @@ static nss_status_t read_gids(
     /* only add the group to the list if it is not the specified group */
     if (gid!=skipgroup)
     {
+#ifdef NSS_FLAVOUR_GLIBC
       /* check if we reached the limit */
       if ( (limit>0) && (*start>=limit) )
         return NSS_STATUS_TRYAGAIN;
       /* check if our buffer is large enough */
-#ifdef NSS_FLAVOUR_GLIBC
       if ((*start)>=(*size))
       {
         /* for some reason Glibc expects us to grow the array (completely
@@ -93,6 +94,14 @@ static nss_status_t read_gids(
         *size=newsize;
       }
 #endif /* NSS_FLAVOUR_GLIBC */
+#ifdef NSS_FLAVOUR_SOLARIS
+      /* check if we reached the limit */
+      if ( (limit>0) && (*start>=limit) )
+      {
+        *errnop=1; /* this is args->erange */
+        return NSS_STATUS_NOTFOUND;
+      }
+#endif /* NSS_FLAVOUR_SOLARIS */
       /* add gid to list */
       (*groupsp)[(*start)++]=gid;
     }
@@ -184,72 +193,43 @@ nss_status_t _nss_ldap_initgroups_dyn(
 #ifdef NSS_FLAVOUR_SOLARIS
 
 #ifdef HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN
-
-static nss_status_t read_groupstring(TFILE *fp,nss_XbyY_args_t *args)
+static char *group2str(struct group *result,char *buffer,size_t buflen)
 {
-  struct group result;
-  nss_status_t retv;
-  char *buffer;
-  size_t buflen;
-  int i;
-  /* read the group into a temporary buffer */
-  buffer=(char *)malloc(args->buf.buflen);
-  if (buffer==NULL)
-    return NSS_STATUS_UNAVAIL;
-  retv=read_group(fp,&result,buffer,args->buf.buflen,&errno);
-  if (retv!=NSS_STATUS_SUCCESS)
-  {
-    free(buffer);
-    return retv;
-  }
-  /* make a string representation */
-  snprintf(args->buf.buffer,args->buf.buflen,
-           "%s:%s:%d:",result.gr_name,result.gr_passwd,(int)result.gr_gid);
-  args->buf.buffer[args->buf.buflen-1]='\0';
-  if (result.gr_mem)
-    for (i=0;result.gr_mem[i];i++)
+  int res,i;
+  res=snprintf(buffer,buflen,"%s:%s:%d:",result->gr_name,result->gr_passwd,(int)result->gr_gid);
+  if ((res<0)||(res>=buflen))
+    return NULL;
+  if (result->gr_mem)
+    for (i=0;result->gr_mem[i];i++)
     {
       if (i)
-        strncat(args->buf.buffer,args->buf.buflen-strlen(args->buf.buffer)-1,",");
-      strncat(args->buf.buffer,args->buf.buflen-strlen(args->buf.buffer)-1,result.gr_mem[i]);
+        strlcat(buffer,",",buflen);
+      strlcat(buffer,result->gr_mem[i],buflen);
     }
-  free(buffer);
   /* check if buffer overflowed */
-  if (strlen(args->buf.buffer)>=args->buf.buffer-1)
-    return NSS_STATUS_TRYAGAIN;
-  NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.buffer;
-  NSS_ARGS(args)->returnlen=strlen(NSS_ARGS(args)->buf.buffer);
-  return NSS_STATUS_SUCCESS;
+  if (strlen(buffer)>=buflen-1)
+    return NULL;
+  return buffer;
 }
+#endif /* HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
 
-#define READ_RESULT(fp) \
-  NSS_ARGS(args)->buf.result? \
-    read_group(fp,(struct group *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno): \
-    read_groupstring(fp,args); \
-  if ((NSS_ARGS(args)->buf.result)&&(retv==NSS_STATUS_SUCCESS)) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#else /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
-
-#define READ_RESULT(fp) \
-  read_group(fp,(struct group *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno); \
-  if (retv==NSS_STATUS_SUCCESS) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#endif /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
+static nss_status_t read_result(TFILE *fp,nss_XbyY_args_t *args)
+{
+  READ_RESULT(group,&args->erange);
+}
 
 static nss_status_t group_getgrnam(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYNAME(NSLCD_ACTION_GROUP_BYNAME,
              NSS_ARGS(args)->key.name,
-             READ_RESULT(fp));
+             read_result(fp,args));
 }
 
 static nss_status_t group_getgrgid(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYTYPE(NSLCD_ACTION_GROUP_BYGID,
              NSS_ARGS(args)->key.gid,gid_t,
-             READ_RESULT(fp));
+             read_result(fp,args));
 }
 
 static nss_status_t group_setgrent(nss_backend_t *be,void UNUSED(*args))
@@ -260,7 +240,7 @@ static nss_status_t group_setgrent(nss_backend_t *be,void UNUSED(*args))
 static nss_status_t group_getgrent(nss_backend_t *be,void *args)
 {
   NSS_GETENT(LDAP_BE(be)->fp,NSLCD_ACTION_GROUP_ALL,
-             READ_RESULT((LDAP_BE(be)->fp)));
+             read_result(LDAP_BE(be)->fp,args));
 }
 
 static nss_status_t group_endgrent(nss_backend_t *be,void UNUSED(*args))
@@ -268,11 +248,6 @@ static nss_status_t group_endgrent(nss_backend_t *be,void UNUSED(*args))
   NSS_ENDENT(LDAP_BE(be)->fp);
 }
 
-/*
-static nss_status_t get_initgroups_dyn(
-        const char *user,gid_t skipgroup,long int *start,
-        gid_t **groupsp,long int limit,int *errnop)
-*/
 static nss_status_t group_getgroupsbymember(nss_backend_t UNUSED(*be),void *args)
 {
   struct nss_groupsbymem *argp=(struct nss_groupsbymem *)args;
@@ -280,7 +255,7 @@ static nss_status_t group_getgroupsbymember(nss_backend_t UNUSED(*be),void *args
   gid_t skipgroup=(start>0)?argp->gid_array[0]:(gid_t)-1;
   NSS_BYNAME(NSLCD_ACTION_GROUP_BYMEMBER,
              argp->username,
-             read_gids(fp,skipgroup,&start,NULL,(gid_t **)&argp->gid_array,argp->maxgids,&errno);
+             read_gids(fp,skipgroup,&start,NULL,(gid_t **)&argp->gid_array,argp->maxgids,&NSS_ARGS(args)->erange);
              argp->numgids=(int)start;);
 }
 
