@@ -2,7 +2,7 @@
    hosts.c - NSS lookup functions for hosts database
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2010 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2010, 2012 Arthur de Jong
    Copyright (C) 2010 Symas Corporation
 
    This library is free software; you can redistribute it and/or
@@ -65,15 +65,16 @@
    specified address family, result is stored in result
    it will an empty entry if no addresses in the address family
    were available */
-static nss_status_t read_hostent(
-        TFILE *fp,int af,struct hostent *result,
-        char *buffer,size_t buflen,int *errnop,int *h_errnop)
+static nss_status_t read_one_hostent(
+        TFILE *fp,struct hostent *result,
+        char *buffer,size_t buflen,int *errnop,int *h_errnop,int af)
 {
   int32_t tmpint32,tmp2int32,tmp3int32;
   int32_t numaddr;
   int i;
   int readaf;
   size_t bufptr=0;
+  memset(result,0,sizeof(struct hostent));
   /* read the host entry */
   READ_BUF_STRING(fp,result->h_name);
   READ_BUF_STRINGLIST(fp,result->h_aliases);
@@ -109,60 +110,33 @@ static nss_status_t read_hostent(
   return NSS_STATUS_SUCCESS;
 }
 
-/* this is a wrapper around read_hostent() that does error handling
-   if the read address list does not contain any addresses for the
-   specified address familiy */
-static nss_status_t read_hostent_erronempty(
-        TFILE *fp,int af,struct hostent *result,
-        char *buffer,size_t buflen,int *errnop,int *h_errnop)
-{
-  nss_status_t retv;
-  retv=read_hostent(fp,af,result,buffer,buflen,errnop,h_errnop);
-  /* check result */
-  if (retv!=NSS_STATUS_SUCCESS)
-    return retv;
-  /* check empty address list
-     (note that we cannot do this in the read_hostent() function as closing
-     the socket there will cause problems with the {set,get,end}ent() functions
-     below)
-  */
-  if (result->h_addr_list[0]==NULL)
-  {
-    *errnop=ENOENT;
-    *h_errnop=NO_ADDRESS;
-    (void)tio_close(fp);
-    return NSS_STATUS_NOTFOUND;
-  }
-  return NSS_STATUS_SUCCESS;
-}
-
-/* this is a wrapper around read_hostent() that skips to the
-   next address if the address list does not contain any addresses for the
-   specified address familiy */
-static nss_status_t read_hostent_nextonempty(
-        TFILE *fp,int af,struct hostent *result,
-        char *buffer,size_t buflen,int *errnop,int *h_errnop)
+/* this is a wrapper around read_one_hostent() that checks whether the read
+   address list is empty and tries the next result if available if
+   retry is set */
+static nss_status_t read_hostent(
+        TFILE *fp,struct hostent *result,
+        char *buffer,size_t buflen,int *errnop,int *h_errnop,int af,int retry)
 {
   int32_t tmpint32;
   nss_status_t retv;
-  /* check until we read an non-empty entry */
-  do
+  /* check until we read an non-empty entry, error or */
+  while (1)
   {
-    /* read a host entry */
-    retv=read_hostent(fp,af,result,buffer,buflen,errnop,h_errnop);
+    retv=read_one_hostent(fp,result,buffer,buflen,errnop,h_errnop,af);
     /* check result */
-    if (retv!=NSS_STATUS_SUCCESS)
+    if ((retv!=NSS_STATUS_SUCCESS)||(result->h_addr_list[0]!=NULL))
       return retv;
-    /* skip to the next entry if we read an empty address */
-    if (result->h_addr_list[0]==NULL)
+    /* error of if we are not retrying */
+    if (!retry)
     {
-      retv=NSS_STATUS_NOTFOUND;
-      READ_RESPONSE_CODE(fp);
+      *errnop=ENOENT;
+      *h_errnop=NO_ADDRESS;
+      (void)tio_close(fp);
+      return NSS_STATUS_NOTFOUND;
     }
-    /* do another loop run if we read an empty address */
+    /* skip to the next entry */
+    READ_RESPONSE_CODE(fp);
   }
-  while (retv!=NSS_STATUS_SUCCESS);
-  return NSS_STATUS_SUCCESS;
 }
 
 /* write an address value */
@@ -186,7 +160,7 @@ nss_status_t _nss_ldap_gethostbyname2_r(
 {
   NSS_BYNAME(NSLCD_ACTION_HOST_BYNAME,
              name,
-             read_hostent_erronempty(fp,af,result,buffer,buflen,errnop,h_errnop));
+             read_hostent(fp,result,buffer,buflen,errnop,h_errnop,af,0));
 }
 
 /* this function just calls the gethostbyname2() variant with the address
@@ -212,7 +186,7 @@ nss_status_t _nss_ldap_gethostbyaddr_r(
 {
   NSS_BYGEN(NSLCD_ACTION_HOST_BYADDR,
             WRITE_ADDRESS(fp,af,len,addr),
-            read_hostent_erronempty(fp,af,result,buffer,buflen,errnop,h_errnop))
+            read_hostent(fp,result,buffer,buflen,errnop,h_errnop,af,0))
 }
 
 /* thread-local file pointer to an ongoing request */
@@ -229,7 +203,7 @@ nss_status_t _nss_ldap_gethostent_r(
         char *buffer,size_t buflen,int *errnop,int *h_errnop)
 {
   NSS_GETENT(hostentfp,NSLCD_ACTION_HOST_ALL,
-             read_hostent_nextonempty(hostentfp,AF_INET,result,buffer,buflen,errnop,h_errnop));
+             read_hostent(hostentfp,result,buffer,buflen,errnop,h_errnop,AF_INET,1));
 }
 
 /* close the stream opened with sethostent() above */
@@ -242,110 +216,58 @@ nss_status_t _nss_ldap_endhostent(void)
 
 #ifdef NSS_FLAVOUR_SOLARIS
 
-struct nss_ldap_hosts_backend
-{
-  nss_backend_op_t *ops;
-  int n_ops;
-  TFILE *fp;
-};
-
 #ifdef HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN
-
-static nss_status_t read_hoststring(TFILE *fp,nss_XbyY_args_t *args,int erronempty)
+static char *hostent2str(struct hostent *result,char *buffer,size_t buflen)
 {
-  struct hostent result;
-  nss_status_t retv;
-  char *buffer;
-  size_t buflen;
-  int i;
-  /* read the hostent */
-  if (erronempty)
-    retv=read_hostent_erronempty(fp,NSS_ARGS(args)->key.hostaddr.type,&result,NSS_ARGS(args)->buf.buffer,args->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno));
-  else
-    retv=read_hostent_nextonempty(fp,NSS_ARGS(args)->key.hostaddr.type,&result,NSS_ARGS(args)->buf.buffer,args->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno));
-  if (retv!=NSS_STATUS_SUCCESS)
-    return retv;
-  /* allocate a temporary buffer */
-  buflen=args->buf.buflen;
-  buffer=(char *)malloc(buflen);
-  /* build the formatted string */
-  /* FIXME: implement proper buffer size checking */
-  if (result.h_addr_list)
+  int i,j;
+  /* build the formatted string, one line per address */
+  buffer[0]='\0';
+  if (result->h_addr_list!=NULL)
   {
-    struct in_addr in;
-    (void)memcpy(&in.s_addr,result.h_addr_list[0],sizeof(in.s_addr));
-    sprintf(buffer,"%s %s",inet_ntoa(in),result.h_name);
-    if (result.h_aliases)
+    for (i=0;result->h_addr_list[i];i++)
     {
-      int j;
-      for (j=0;result.h_aliases[j];j++)
+      if (i>0)
+        strlcat(buffer,"\n",buflen);
+      /* snprintf writes a terminating \0 on Solaris */
+      snprintf(buffer,buflen-strlen(buffer)-1,
+               "%s %s",inet_ntoa(*((struct in_addr *)result->h_addr_list[i])),result->h_name);
+      /* add aliases for first line only */
+      if ((i==0)&&(result->h_aliases))
       {
-        strcat(buffer,"  ");
-        strcat(buffer,result.h_aliases[j]);
+        for (j=0;result->h_aliases[j];j++)
+        {
+          strlcat(buffer," ",buflen);
+          strlcat(buffer,result->h_aliases[j],buflen);
+        }
       }
     }
-    for (i=1;result.h_addr_list[i];i++)
-    {
-      (void)memcpy(&in.s_addr,result.h_addr_list[i],sizeof(in.s_addr));
-      strcat(buffer,"\n");
-      strcat(buffer,inet_ntoa(in));
-      strcat(buffer," ");
-      strcat(buffer,result.h_name);
-      /* TODO: aliases only supplied to the first address */
-      /* need review */
-    }
   }
-  /* copy the result back to the result buffer and free the temporary one */
-  strcpy(NSS_ARGS(args)->buf.buffer,buffer);
-  free(buffer);
-  NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.buffer;
-  NSS_ARGS(args)->returnlen=strlen(NSS_ARGS(args)->buf.buffer);
-  return NSS_STATUS_SUCCESS;
+  if (strlen(buffer)>=buflen-1)
+    return NULL;
+  return buffer;
+}
+#endif /* HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
+
+static nss_status_t read_result(TFILE *fp,int af,int retry,nss_XbyY_args_t *args)
+{
+  READ_RESULT(hostent,&args->erange,&args->h_errno,af,retry);
 }
 
-#define READ_RESULT_ERRONEMPTY(fp) \
-  NSS_ARGS(args)->buf.result? \
-    read_hostent_erronempty(fp,NSS_ARGS(args)->key.hostaddr.type,(struct hostent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno)): \
-    read_hoststring(fp,args,1); \
-  if ((NSS_ARGS(args)->buf.result)&&(retv==NSS_STATUS_SUCCESS)) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#define READ_RESULT_NEXTONEMPTY(fp) \
-  NSS_ARGS(args)->buf.result? \
-    read_hostent_nextonempty(fp,NSS_ARGS(args)->key.hostaddr.type,(struct hostent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno)): \
-    read_hoststring(fp,args,0); \
-  if ((NSS_ARGS(args)->buf.result)&&(retv==NSS_STATUS_SUCCESS)) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#else /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
-
-#define READ_RESULT_ERRONEMPTY(fp) \
-  read_hostent_erronempty(fp,NSS_ARGS(args)->key.hostaddr.type,(struct hostent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno)); \
-  if (retv==NSS_STATUS_SUCCESS) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#define READ_RESULT_NEXTONEMPTY(fp) \
-  read_hostent_nextonempty(fp,NSS_ARGS(args)->key.hostaddr.type,(struct hostent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno)); \
-  if (retv==NSS_STATUS_SUCCESS) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#endif /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
-
-/* hack to set the correct errno and h_errno */
+/* hack to set the correct h_errno */
 #define h_errnop &(NSS_ARGS(args)->h_errno)
 
 static nss_status_t hosts_gethostbyname(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYNAME(NSLCD_ACTION_HOST_BYNAME,
              NSS_ARGS(args)->key.name,
-             READ_RESULT_ERRONEMPTY(fp));
+             read_result(fp,AF_INET,0,args));
 }
 
 static nss_status_t hosts_gethostbyaddr(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYGEN(NSLCD_ACTION_HOST_BYADDR,
             WRITE_ADDRESS(fp,NSS_ARGS(args)->key.hostaddr.type,NSS_ARGS(args)->key.hostaddr.len,NSS_ARGS(args)->key.hostaddr.addr),
-            READ_RESULT_ERRONEMPTY(fp));
+            read_result(fp,NSS_ARGS(args)->key.hostaddr.type,0,args));
 }
 
 static nss_status_t hosts_sethostent(nss_backend_t *be,void UNUSED(*args))
@@ -356,7 +278,7 @@ static nss_status_t hosts_sethostent(nss_backend_t *be,void UNUSED(*args))
 static nss_status_t hosts_gethostent(nss_backend_t *be,void *args)
 {
   NSS_GETENT(LDAP_BE(be)->fp,NSLCD_ACTION_HOST_ALL,
-             READ_RESULT_NEXTONEMPTY(LDAP_BE(be)->fp));
+             read_result(LDAP_BE(be)->fp,AF_INET,1,args));
 }
 
 static nss_status_t hosts_endhostent(nss_backend_t *be,void UNUSED(*args))
