@@ -2,7 +2,7 @@
    networks.c - NSS lookup functions for networks database
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2010, 2011 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2010, 2011, 2012 Arthur de Jong
    Copyright (C) 2010 Symas Corporation
 
    This library is free software; you can redistribute it and/or
@@ -72,6 +72,7 @@ static nss_status_t read_netent(
   int readaf;
   size_t bufptr=0;
   nss_status_t retv=NSS_STATUS_NOTFOUND;
+  memset(result,0,sizeof(struct netent));
   /* read the network entry */
   READ_BUF_STRING(fp,result->n_name);
   READ_BUF_STRINGLIST(fp,result->n_aliases);
@@ -162,55 +163,30 @@ nss_status_t _nss_ldap_endnetent(void)
 #ifdef NSS_FLAVOUR_SOLARIS
 
 #ifdef HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN
-
-static nss_status_t read_netentstring(TFILE *fp,nss_XbyY_args_t *args)
+static char *netent2str(struct netent *result,char *buffer,size_t buflen)
 {
-  struct netent result;
-  nss_status_t retv;
-  char *buffer;
-  size_t buflen;
-  int i;
+  int i,res;
   struct in_addr priv_in_addr;
-  /* read the netent */
-  retv=read_netent(fp,&result,NSS_ARGS(args)->buf.buffer,args->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno));
-  if (retv!=NSS_STATUS_SUCCESS)
-    return retv;
-  /* allocate a temporary buffer */
-  buflen=args->buf.buflen;
-  buffer=(char *)malloc(buflen);
-  /* build the formatted string */
-  /* FIXME: implement proper buffer size checking */
-  priv_in_addr.s_addr = result.n_net;
-  sprintf(buffer,"%s %s",result.n_name,inet_ntoa(priv_in_addr)); /* ipNetworkNumber */
-  if (result.n_aliases)
-    for (i=0;result.n_aliases[i];i++)
+  priv_in_addr.s_addr=result->n_net;
+  res=snprintf(buffer,buflen,"%s %s",result->n_name,inet_ntoa(priv_in_addr));
+  if ((res<0)||(res>=buflen))
+    return NULL;
+  if (result->n_aliases)
+    for (i=0;result->n_aliases[i];i++)
     {
-      strcat(buffer," ");
-      strcat(buffer,result.n_aliases[i]);
+      strlcat(buffer," ",buflen);
+      strlcat(buffer,result->n_aliases[i],buflen);
     }
-  /* copy the result back to the result buffer and free the temporary one */
-  strcpy(NSS_ARGS(args)->buf.buffer,buffer);
-  free(buffer);
-  NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.buffer;
-  NSS_ARGS(args)->returnlen=strlen(NSS_ARGS(args)->buf.buffer);
-  return NSS_STATUS_SUCCESS;
+  if (strlen(buffer)>=buflen-1)
+    return NULL;
+  return buffer;
 }
+#endif /* HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
 
-#define READ_RESULT(fp) \
-  NSS_ARGS(args)->buf.result? \
-    read_netent(fp,(struct netent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno)): \
-    read_netentstring(fp,args); \
-  if ((NSS_ARGS(args)->buf.result)&&(retv==NSS_STATUS_SUCCESS)) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#else /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
-
-#define READ_RESULT(fp) \
-  read_netent(fp,(struct netent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno,&(NSS_ARGS(args)->h_errno)); \
-  if (retv==NSS_STATUS_SUCCESS) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#endif /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
+static nss_status_t read_result(TFILE *fp,nss_XbyY_args_t *args)
+{
+  READ_RESULT(netent,&args->erange,&args->h_errno);
+}
 
 /* more of a dirty hack */
 #define h_errnop (&(NSS_ARGS(args)->h_errno))
@@ -219,14 +195,14 @@ static nss_status_t networks_getnetbyname(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYNAME(NSLCD_ACTION_NETWORK_BYNAME,
              NSS_ARGS(args)->key.name,
-             READ_RESULT(fp));
+             read_result(fp,args));
 }
 
 static nss_status_t networks_getnetbyaddr(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYGEN(NSLCD_ACTION_NETWORK_BYADDR,
             WRITE_ADDRESS(fp,NSS_ARGS(args)->key.netaddr.net),
-            READ_RESULT(fp));
+            read_result(fp,args));
 }
 
 static nss_status_t networks_setnetent(nss_backend_t *be,void UNUSED(*args))
@@ -237,7 +213,7 @@ static nss_status_t networks_setnetent(nss_backend_t *be,void UNUSED(*args))
 static nss_status_t networks_getnetent(nss_backend_t *be,void *args)
 {
   NSS_GETENT(LDAP_BE(be)->fp,NSLCD_ACTION_NETWORK_ALL,
-             READ_RESULT(LDAP_BE(be)->fp));
+             read_result(LDAP_BE(be)->fp,args));
 }
 
 static nss_status_t networks_endnetent(nss_backend_t *be,void UNUSED(*args))
