@@ -2,7 +2,7 @@
    passwd.c - NSS lookup functions for passwd database
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2010 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2010, 2012 Arthur de Jong
    Copyright (C) 2010 Symas Corporation
 
    This library is free software; you can redistribute it and/or
@@ -37,6 +37,7 @@ static nss_status_t read_passwd(
 {
   int32_t tmpint32;
   size_t bufptr=0;
+  memset(result,0,sizeof(struct passwd));
   READ_BUF_STRING(fp,result->pw_name);
   READ_BUF_STRING(fp,result->pw_passwd);
   READ_TYPE(fp,result->pw_uid,uid_t);
@@ -44,6 +45,10 @@ static nss_status_t read_passwd(
   READ_BUF_STRING(fp,result->pw_gecos);
   READ_BUF_STRING(fp,result->pw_dir);
   READ_BUF_STRING(fp,result->pw_shell);
+#ifdef HAVE_STRUCT_PASSWD_PW_CLASS
+  /* set the user access class to an empty string */
+  result->pw_class=result->pw_name+strlen(result->pw_name);
+#endif /* HAVE_STRUCT_PASSWD_PW_CLASS */
   return NSS_STATUS_SUCCESS;
 }
 
@@ -98,61 +103,35 @@ nss_status_t _nss_ldap_endpwent(void)
 #ifdef NSS_FLAVOUR_SOLARIS
 
 #ifdef HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN
-
-static nss_status_t read_passwdstring(TFILE *fp,nss_XbyY_args_t *args)
+static char *passwd2str(struct passwd *result,char *buffer,size_t buflen)
 {
-  struct passwd result;
-  nss_status_t retv;
-  char *buffer;
-  size_t buflen;
-  /* read the passwd */
-  retv=read_passwd(fp,&result,NSS_ARGS(args)->buf.buffer,args->buf.buflen,&errno);
-  if (retv!=NSS_STATUS_SUCCESS)
-    return retv;
-  /* allocate a temporary buffer */
-  buflen=args->buf.buflen;
-  buffer=(char *)malloc(buflen);
-  /* build the formatted string */
-  /* FIXME: implement proper buffer size checking */
-   sprintf(buffer,"%s:%s:%d:%d:%s:%s:%s",
-     result.pw_name,result.pw_passwd,(int)result.pw_uid,(int)result.pw_gid,result.pw_gecos,
-     result.pw_dir,result.pw_shell);
-  /* copy the result back to the result buffer and free the temporary one */
-  strcpy(NSS_ARGS(args)->buf.buffer,buffer);
-  free(buffer);
-  NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.buffer;
-  NSS_ARGS(args)->returnlen=strlen(NSS_ARGS(args)->buf.buffer);
-  return NSS_STATUS_SUCCESS;
+  int res;
+  res=snprintf(buffer,buflen,"%s:%s:%d:%d:%s:%s:%s",
+     result->pw_name,result->pw_passwd,(int)result->pw_uid,(int)result->pw_gid,
+     result->pw_gecos,result->pw_dir,result->pw_shell);
+  if ((res<0)||(res>=buflen))
+    return NULL;
+  return buffer;
 }
+#endif /* HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
 
-#define READ_RESULT(fp) \
-  NSS_ARGS(args)->buf.result? \
-    read_passwd(fp,(struct passwd *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno): \
-    read_passwdstring(fp,args); \
-  if ((NSS_ARGS(args)->buf.result)&&(retv==NSS_STATUS_SUCCESS)) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#else /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
-
-#define READ_RESULT(fp) \
-  read_passwd(fp,(struct passwd *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno); \
-  if (retv==NSS_STATUS_SUCCESS) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#endif /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
+static nss_status_t read_result(TFILE *fp,nss_XbyY_args_t *args)
+{
+  READ_RESULT(passwd,&args->erange);
+}
 
 static nss_status_t passwd_getpwnam(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYNAME(NSLCD_ACTION_PASSWD_BYNAME,
              NSS_ARGS(args)->key.name,
-             READ_RESULT(fp));
+             read_result(fp,args));
 }
 
 static nss_status_t passwd_getpwuid(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYTYPE(NSLCD_ACTION_PASSWD_BYUID,
              NSS_ARGS(args)->key.uid,uid_t,
-             READ_RESULT(fp));
+             read_result(fp,args));
 }
 
 /* open a connection to the nslcd and write the request */
@@ -165,7 +144,7 @@ static nss_status_t passwd_setpwent(nss_backend_t *be,void UNUSED(*args))
 static nss_status_t passwd_getpwent(nss_backend_t *be,void *args)
 {
   NSS_GETENT(LDAP_BE(be)->fp,NSLCD_ACTION_PASSWD_ALL,
-             READ_RESULT(LDAP_BE(be)->fp));
+             read_result(LDAP_BE(be)->fp,args));
 }
 
 /* close the stream opened with setpwent() above */
