@@ -2,7 +2,7 @@
    service.c - NSS lookup functions for services database
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007, 2008, 2010 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008, 2010, 2012 Arthur de Jong
    Copyright (C) 2010 Symas Corporation
 
    This library is free software; you can redistribute it and/or
@@ -37,6 +37,7 @@ static nss_status_t read_servent(
 {
   int32_t tmpint32,tmp2int32,tmp3int32;
   size_t bufptr=0;
+  memset(result,0,sizeof(struct servent));
   READ_BUF_STRING(fp,result->s_name);
   READ_BUF_STRINGLIST(fp,result->s_aliases);
   /* store port number in network byte order */
@@ -98,60 +99,35 @@ nss_status_t _nss_ldap_endservent(void)
 #ifdef NSS_FLAVOUR_SOLARIS
 
 #ifdef HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN
-
-static nss_status_t read_servstring(TFILE *fp,nss_XbyY_args_t *args)
+static char *servent2str(struct servent *result,char *buffer,size_t buflen)
 {
-  struct servent result;
-  nss_status_t retv;
-  char *buffer;
-  size_t buflen;
-  int i;
-  /* read the servent */
-  retv=read_servent(fp,&result,NSS_ARGS(args)->buf.buffer,args->buf.buflen,&errno);
-  if (retv!=NSS_STATUS_SUCCESS)
-    return retv;
-  /* allocate a temporary buffer */
-  buflen=args->buf.buflen;
-  buffer=(char *)malloc(buflen);
-  /* build the formatted string */
-  /* FIXME: implement proper buffer size checking */
-  sprintf(buffer,"%s %d/%s",result.s_name,result.s_port,result.s_proto);
-  if (result.s_aliases)
-    for (i=0;result.s_aliases[i];i++)
+  int res,i;
+  res=snprintf(buffer,buflen,"%s %d/%s",result->s_name,result->s_port,result->s_proto);
+  if ((res<0)||(res>=buflen))
+    return NULL;
+  if (result->s_aliases)
+    for (i=0;result->s_aliases[i];i++)
     {
-      strcat(buffer," ");
-      strcat(buffer,result.s_aliases[i]);
+      strlcat(buffer," ",buflen);
+      strlcat(buffer,result->s_aliases[i],buflen);
     }
-  /* copy the result back to the result buffer and free the temporary one */
-  strcpy(NSS_ARGS(args)->buf.buffer,buffer);
-  free(buffer);
-  NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.buffer;
-  NSS_ARGS(args)->returnlen=strlen(NSS_ARGS(args)->buf.buffer);
-  return NSS_STATUS_SUCCESS;
+  if (strlen(buffer)>=buflen-1)
+    return NULL;
+  return buffer;
 }
+#endif /* HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
 
-#define READ_RESULT(fp) \
-  NSS_ARGS(args)->buf.result? \
-    read_servent(fp,(struct servent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno): \
-    read_servstring(fp,args); \
-  if ((NSS_ARGS(args)->buf.result)&&(retv==NSS_STATUS_SUCCESS)) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#else /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
-
-#define READ_RESULT(fp) \
-  read_servent(fp,(struct servent *)NSS_ARGS(args)->buf.result,NSS_ARGS(args)->buf.buffer,NSS_ARGS(args)->buf.buflen,&errno); \
-  if (retv==NSS_STATUS_SUCCESS) \
-    NSS_ARGS(args)->returnval=NSS_ARGS(args)->buf.result;
-
-#endif /* not HAVE_STRUCT_NSS_XBYY_ARGS_RETURNLEN */
+static nss_status_t read_result(TFILE *fp,nss_XbyY_args_t *args)
+{
+  READ_RESULT(servent,&args->erange);
+}
 
 static nss_status_t services_getservbyname(nss_backend_t UNUSED(*be),void *args)
 {
   NSS_BYGEN(NSLCD_ACTION_SERVICE_BYNAME,
             WRITE_STRING(fp,NSS_ARGS(args)->key.serv.serv.name);
             WRITE_STRING(fp,NSS_ARGS(args)->key.serv.proto),
-            READ_RESULT(fp));
+            read_result(fp,args));
 }
 
 static nss_status_t services_getservbyport(nss_backend_t UNUSED(*be),void *args)
@@ -159,7 +135,7 @@ static nss_status_t services_getservbyport(nss_backend_t UNUSED(*be),void *args)
   NSS_BYGEN(NSLCD_ACTION_SERVICE_BYNUMBER,
             WRITE_INT32(fp,ntohs(NSS_ARGS(args)->key.serv.serv.port));
             WRITE_STRING(fp,NSS_ARGS(args)->key.serv.proto),
-            READ_RESULT(fp));
+            read_result(fp,args));
 }
 
 static nss_status_t services_setservent(nss_backend_t *be,void UNUSED(*args))
@@ -170,7 +146,7 @@ static nss_status_t services_setservent(nss_backend_t *be,void UNUSED(*args))
 static nss_status_t services_getservent(nss_backend_t *be,void *args)
 {
   NSS_GETENT(LDAP_BE(be)->fp,NSLCD_ACTION_SERVICE_ALL,
-             READ_RESULT(LDAP_BE(be)->fp));
+             read_result(LDAP_BE(be)->fp,args));
 }
 
 static nss_status_t services_endservent(nss_backend_t *be,void UNUSED(*args))
