@@ -1,7 +1,7 @@
 
 # ether.py - lookup functions for ethernet addresses
 #
-# Copyright (C) 2010, 2011 Arthur de Jong
+# Copyright (C) 2010, 2011, 2012 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -20,8 +20,9 @@
 
 import struct
 
-import constants
+import cache
 import common
+import constants
 
 
 def ether_aton(ether):
@@ -46,18 +47,20 @@ class Search(common.Search):
     required = ('cn', 'macAddress')
 
 
+class Cache(cache.Cache):
+    pass
+
+
 class EtherRequest(common.Request):
 
-    def write(self, dn, attributes, parameters):
-        # get values
-        names = attributes['cn']
-        addresses = [ether_aton(x) for x in attributes['macAddress']]
-        # write results
-        for name in names:
-            for ether in addresses:
-                self.fp.write_int32(constants.NSLCD_RESULT_BEGIN)
-                self.fp.write_string(name)
-                self.fp.write(ether)
+    def write(self, name, ether):
+        self.fp.write_string(name)
+        self.fp.write(ether_aton(ether))
+
+    def convert(self, dn, attributes, parameters):
+        for name in attributes['cn']:
+            for ether in attributes['macAddress']:
+                yield (name, ether)
 
 
 class EtherByNameRequest(EtherRequest):
