@@ -1,7 +1,7 @@
 
 # group.py - group entry lookup routines
 #
-# Copyright (C) 2010, 2011 Arthur de Jong
+# Copyright (C) 2010, 2011, 2012 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -18,12 +18,13 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301 USA
 
+import itertools
 import logging
-import ldap.filter
 
-import constants
-import common
 from passwd import dn2uid, uid2dn
+import cache
+import common
+import constants
 
 
 def clean(lst):
@@ -60,16 +61,38 @@ class Search(common.Search):
             dn = uid2dn(self.conn, memberuid)
             if dn:
                 return '(&%s(|(%s=%s)(%s=%s)))' % (self.filter,
-                          attmap['memberUid'], ldap.filter.escape_filter_chars(memberuid),
-                          attmap['member'], ldap.filter.escape_filter_chars(dn))
+                          attmap['memberUid'], self.escape(memberuid),
+                          attmap['member'], self.escape(dn))
         return super(Search, self).mk_filter()
 
 
+class Cache(cache.Cache):
+
+    retrieve_sql = '''
+        SELECT `cn`, `userPassword`, `gidNumber`, `memberUid`
+        FROM `group_cache`
+        LEFT JOIN `group_3_cache`
+          ON `group_3_cache`.`group` = `group_cache`.`cn`
+        '''
+
+    def retrieve(self, parameters):
+        query = cache.Query(self.retrieve_sql, parameters)
+        # return results returning the members as a set
+        q = itertools.groupby(query.execute(self.con),
+                key=lambda x: (x['cn'], x['userPassword'], x['gidNumber']))
+        for k, v in q:
+            yield k + (set(x['memberUid'] for x in v if x['memberUid'] is not None), )
+
+
 class GroupRequest(common.Request):
 
-    wantmembers = True
+    def write(self, name, passwd, gid, members):
+        self.fp.write_string(name)
+        self.fp.write_string(passwd)
+        self.fp.write_gid_t(gid)
+        self.fp.write_stringlist(members)
 
-    def write(self, dn, attributes, parameters):
+    def convert(self, dn, attributes, parameters):
         # get group names and check against requested group name
         names = attributes['cn']
         # get group group password
@@ -78,27 +101,23 @@ class GroupRequest(common.Request):
         gids = [int(x) for x in attributes['gidNumber']]
         # build member list
         members = set()
-        if self.wantmembers:
-            # add the memberUid values
-            for member in clean(attributes['memberUid']):
-                if common.isvalidname(member):
-                    members.add(member)
-            # translate and add the member values
-            for memberdn in clean(attributes['member']):
-                member = dn2uid(self.conn, memberdn)
-                if member and common.isvalidname(member):
-                    members.add(member)
+        # add the memberUid values
+        for member in clean(attributes['memberUid']):
+            if common.isvalidname(member):
+                members.add(member)
+        # translate and add the member values
+        for memberdn in clean(attributes['member']):
+            member = dn2uid(self.conn, memberdn)
+            if member and common.isvalidname(member):
+                members.add(member)
         # actually return the results
         for name in names:
             if not common.isvalidname(name):
-                print '%s: %s: denied by validnames option' % (dn, attmap['cn'])
+                logging.warning('%s: %s: denied by validnames option', dn,
+                                attmap['cn'])
             else:
                 for gid in gids:
-                    self.fp.write_int32(constants.NSLCD_RESULT_BEGIN)
-                    self.fp.write_string(name)
-                    self.fp.write_string(passwd)
-                    self.fp.write_gid_t(gid)
-                    self.fp.write_stringlist(members)
+                    yield (name, passwd, gid, members)
 
 
 class GroupByNameRequest(GroupRequest):
@@ -122,7 +141,6 @@ class GroupByGidRequest(GroupRequest):
 class GroupByMemberRequest(GroupRequest):
 
     action = constants.NSLCD_ACTION_GROUP_BYMEMBER
-    wantmembers = False
 
     def read_parameters(self, fp):
         memberuid = fp.read_string()
