@@ -1,7 +1,7 @@
 
 # network.py - lookup functions for network names and addresses
 #
-# Copyright (C) 2011 Arthur de Jong
+# Copyright (C) 2011, 2012 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -18,10 +18,9 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 # 02110-1301 USA
 
-import logging
-
-import constants
+import cache
 import common
+import constants
 
 
 attmap = common.Attributes(cn='cn',
@@ -35,21 +34,44 @@ class Search(common.Search):
     required = ('cn', )
 
 
+class NetworkQuery(cache.CnAliasedQuery):
+
+    sql = '''
+        SELECT `network_cache`.`cn` AS `cn`,
+               `network_1_cache`.`cn` AS `alias`,
+               `network_2_cache`.`ipNetworkNumber` AS `ipNetworkNumber`
+        FROM `network_cache`
+        LEFT JOIN `network_1_cache`
+          ON `network_1_cache`.`network` = `network_cache`.`cn`
+        LEFT JOIN `network_2_cache`
+          ON `network_2_cache`.`network` = `network_cache`.`cn`
+        '''
+
+    def __init__(self, parameters):
+        super(NetworkQuery, self).__init__('network', parameters)
+
+
+class Cache(cache.Cache):
+
+    def retrieve(self, parameters):
+        query = NetworkQuery(parameters)
+        for row in cache.RowGrouper(query.execute(self.con), ('cn', ), ('alias', 'ipNetworkNumber', )):
+            yield row['cn'], row['alias'], row['ipNetworkNumber']
+
+
 class NetworkRequest(common.Request):
 
-    def write(self, dn, attributes, parameters):
-        # get values
-        networknames = attributes['cn']
-        networkname = networknames.pop(0)
-        addresses = attributes['ipNetworkNumber']
-        # write result
-        self.fp.write_int32(constants.NSLCD_RESULT_BEGIN)
+    def write(self, networkname, aliases, addresses):
         self.fp.write_string(networkname)
-        self.fp.write_stringlist(networknames)
+        self.fp.write_stringlist(aliases)
         self.fp.write_int32(len(addresses))
         for address in addresses:
             self.fp.write_address(address)
 
+    def convert(self, dn, attributes, parameters):
+        netnames = attributes['cn']
+        yield (netnames[0], netnames[1:], attributes['ipNetworkNumber'])
+
 
 class NetworkByNameRequest(NetworkRequest):
 
